# -*- Mode: cperl -*-
#--------------------------------------------------------------------
# Copyright (C) 2000, 2001, 2002 by MandrakeSoft.
# Chmouel Boudjnah <chmouel@mandrakesoft.com>.
#
# Redistribution of this file is permitted under the terms of the GNU
# Public License (GPL)
#--------------------------------------------------------------------
# $Id: common.pm,v 1.51 2004/03/02 10:36:02 nplanel Exp $
#--------------------------------------------------------------------

package common;
use strict;
use MDK::Common;

my $debug = 0;

my $sysconfig = $debug ? "./installkernel.sysconfig" : "/etc/sysconfig/installkernel";
my $sysconfig_theme = "/etc/sysconfig/bootsplash";

my $mdksub = "smp|enterprise|secure|linus|mosix|BOOT|custom|i686-up-4GB|p3-smp-64GB";

sub check_default_entry {
    # dont ask me why i need to do that :p
    my $foo = $_[0];
    my %entry = %$foo;
    my $foo1 = $_[1];
    my %main = %$foo1;

    # check if the default entry if there is an initrd pointing to the
    # right kernel.
    while (my $act = each %entry) {

	# find the default ?
	if ($act =~ /^$main{default}$/) {
	    # there is an initrd ?
	    if ($entry{$act}{initrd}) {

		my $initrd_version_of_entry;
		my $kernel_version_of_entry;

		#too much code :p but i see problems coming and more
		#different code between the two case

		my $initrd_of_entry = $entry{$act}{initrd};
		$initrd_of_entry = readlink($initrd_of_entry) if -l $initrd_of_entry;

		#A mdk kernel so make sure to get only the mdk one
		if ($initrd_of_entry =~ /mdk/ && $initrd_of_entry =~ m,(/boot/)?initrd-(.*)mdk($mdksub|.*)?\.img?,) {
		    my $s = $3;
		    $initrd_version_of_entry = $2 . "mdk" if $2;
		    #<chmou dark-satanique mode>
		    #	That tricky since if it is a mdk kernel it should
		    #	match $mdksub so since here if it's don't
		    #	match we don't add to the
		    #	$initrd_version_of_entry which mean wrong name
		    #	and fail.
		    #</chmou dark-satanique mode>
		    $initrd_version_of_entry .= $s if $s =~ /$mdksub/;
		}

		my $kernel_of_entry = $entry{$act}{kernel};
		$kernel_of_entry = readlink($kernel_of_entry) if -l $kernel_of_entry;

		#A mdk kernel so make sure to get only the mdk one
		if ($kernel_of_entry =~ /mdk/ && $kernel_of_entry =~ m,(/boot/)?vmlinuz-(.*)mdk($mdksub|.*)?,) {
		    my $s = $3;
		    $kernel_version_of_entry = $2 . "mdk" if $2;
		    if ($s && $s !~ /$mdksub/) {
			print "Looks like you have an unknown mdk kernel, please contact Juan Quintela <quintela\@mandrakesoft.com>\n";
			# We still continue and don't make it fail
			# fail, since we want to add a new kernel
			# anyway
			return;
		    }
		    $kernel_version_of_entry .= $s if $s;
		}

		print "kernel=$kernel_version_of_entry, initrd=$initrd_version_of_entry\n" if $debug;
		if ($kernel_version_of_entry !~ /^$initrd_version_of_entry$/) {
		    print STDERR ("looks like there is a problem, the default vmlinuz version is not the same \nas the initrd\n");
		    print STDERR ("which means you have a mdk kernel and not a mdk initrd wich may give you trouble\n");
		    return 1;
		}
	    } else { #check without initrd
		my $kernel_of_entry = $entry{$act}{kernel};
		$kernel_of_entry = readlink($kernel_of_entry) if -l $kernel_of_entry;

		#A mdk kernel so make sure to get only the mdk one
		if ($kernel_of_entry =~ /mdk/ && $kernel_of_entry =~ m,(/boot/)?vmlinuz-(.*)mdk($mdksub|.*)?,) {
		    my $s = $3;
		    if ($s && $s !~ /$mdksub/) {
			print "Looks like you have an unknown mdk kernel => $kernel_of_entry\n";
			print "please send a mail on Mandrake Cooker <cooker\@linux-mandrake.com>\n";
			# We still continue and don't make it fail
			# fail, since we want to add a new kernel
			# anyway
			return;
		    }
		}
	    }
	}
     }
}

sub get_first_boot {
    my ($_type, $resolved_link) = @_;
    my $boot = '/boot/';
    my @list;
    local *D;
    opendir D, $boot;
    foreach (readdir(D)) {
	if (/vmlinuz-/i && !/$resolved_link/ && !/vmlinuz-($mdksub)/) {
	    push @list, $_
	}
    }
    close D;
    @list = reverse(sort(@list));
    (my $first_kernel = shift(@list)) =~ s|vmlinuz-||;
    unless (-f "/boot/initrd-$first_kernel.img") {
	system("perl /usr/share/loader/make-initrd $first_kernel") unless $debug;
    }
    return $first_kernel;
}

sub do_check {
    my ($label, $version, $remove) = @_;
    my $entry = sanitize_ver($version);
    if ($remove && !$label) {
	print("not removing bootloader entry: $entry, it doesn't exist\n(maybe it has been renamed...)\n");
	exit();
    }
    if ($label && !$remove) {
	print("not adding bootloader entry: $label, it already exists\n");
	exit();
    }
    unless (-f "/boot/vmlinuz-$version") {
	print("looks like I cannot find the specified kernel /boot/vmlinuz-$version\n") ;
	exit() if !$debug;
    }
}

sub symlinkf { unlink $_[1]; symlink $_[0], $_[1] }
sub cat_ { local *F; open(F, $_[0]) or ($_[1] ? die "cat of file $_[0] failed: $!\n" : return); my @l = <F>; wantarray() ? @l : join '', @l }

sub getoptions {
    my $lilo = shift;
    my $options;
    my $quiet = getquiet_from_themes();
    my %var = getVarsFromSh($sysconfig);
    my $cmdline = join('', cat_("/proc/cmdline")) if !any { /NOCMDLINE/ } $var{OPTIONS};
    foreach my $foo (split(/\s+/, join(" ", $var{OPTIONS}, $cmdline)))  {
	next if $foo =~ /auto/ && $lilo;
	next if $foo =~ /root=/;
	next if $foo =~ /quiet/ && $quiet;
	next if $foo =~ /^r[ow]/;
	next if $foo =~ /BOOT_IMAGE=/;
	next if $foo =~ /NOCMDLINE/;
	next if $foo =~ /=ide-scsi/;
	next if any { /$foo/ } $options;
	$options .= "$foo ";
    }
    return $options;
}


#Getroot(): First try to get root= from sysconfig files, if we don't
#	found it try to discover in the /proc/cmdline, if there is not
#	(like when booting from lilo that put the root in hexa
#	numbers) try to detect from fstab and if we didn't find it (no
#	s how) get it from rdev.

sub getroot() {
    my $root;
    $root = getroot_from_sysconfig();
    $root ||= (grep { m!root=/\S+! } split(" ", cat_('/proc/cmdline')))[0];
    $root ||= getroot_from_fstab();
    ($root = `rdev`) =~ s|(\S+)\s+.*$|root=$1| unless $root;chomp $root;
    warn("There is no root= defined, you will have trouble\n") unless $root;
    return $root;
}

sub getroot_from_sysconfig() {
    my %var = getVarsFromSh($sysconfig);
    return (grep { m!/root=/\S+! } split(" ", $var{OPTIONS}))[0];
}

sub gettheme_from_sysconfig() {
    my %var = getVarsFromSh($sysconfig_theme);
    return $var{THEME} || "Mandrake";
}

sub getquiet_from_themes() {
    return unless -f $sysconfig_theme;
    my $theme_config = "/usr/share/bootsplash/themes/" . gettheme_from_sysconfig() . "/global.config";
    my %var = getVarsFromSh($theme_config);
    return 1 if $var{QUIET} =~ /^yes$/i;
    return;
}

sub getroot_from_fstab() {
    my $part;
    local *F;
    open F, '/etc/fstab';
    while (<F>) {
	my @s = split ' ';
	$part = $s[0] if $s[1] =~ m|/$| && !$part;
    }
    close F;
    return "root=" . $part;
}

sub get_kernel_type {
    my ($arg) = @_;
    return $1 if $arg =~ /.*($mdksub)$/;
    return undef;
}

# sanitize_ver: long function when it could be shorter but we are sure
#		to catch everything and can be readable if we want to
#		add new scheme name.
# DUPLICATED in DrakX gi/perl-install/bootloader.pm
sub sanitize_ver {
    my ($string) = @_;

    my ($main_version, undef, $extraversion, $rest) = 
      $string =~ m!(\d+\.\d+\.\d+)(-((?:pre|rc)\d+))?(.*)!;

    if (my ($mdkver, $cpu, $nproc, $mem) = $rest =~ m|-(.+)-(.+)-(.+)-(.+)|) {
	$rest = "$cpu$nproc$mem-$mdkver";
    }

    my $return = "$main_version$extraversion$rest";

    $return =~ s|\.||g;
    $return =~ s|mdk||;
    $return =~ s|64GB|64G|;
    $return =~ s|4GB|4G|;
    $return =~ s|secure|sec|;
    $return =~ s|enterprise|ent|;

    return $return;
}

1;#wonderful perl
